@@ -109,7 +109,7 @@
 #define LLVM_HAS_SPARC_TARGET 1
 
 /* Define if the SPIRV target is built in */
-#define LLVM_HAS_SPIRV_TARGET 0
+#define LLVM_HAS_SPIRV_TARGET 1
 
 /* Define if the SystemZ target is built in */
 #define LLVM_HAS_SYSTEMZ_TARGET 1
@@ -142,7 +142,7 @@
 #define LLVM_USE_PERF 1
 
 /* Major version of the LLVM API */
-#define LLVM_VERSION_MAJOR 18
+#define LLVM_VERSION_MAJOR 20
 
 /* Minor version of the LLVM API */
 #define LLVM_VERSION_MINOR 1
@@ -151,7 +151,7 @@
 #define LLVM_VERSION_PATCH 8
 
 /* LLVM version string */
-#define LLVM_VERSION_STRING "18.1.8"
+#define LLVM_VERSION_STRING "20.1.8"
 
 /* Whether LLVM records statistics for use with GetStatistics(),
  * PrintStatistics() or PrintStatisticsJSON()
@@ -162,7 +162,7 @@
 /* #undef LLVM_WITH_Z3 */
 
 /* Define if we have curl and want to use it */
-/* #undef LLVM_ENABLE_CURL */
+#define LLVM_ENABLE_CURL 1
 
 /* Define if we have cpp-httplib and want to use it */
 /* #undef LLVM_ENABLE_HTTPLIB */
@@ -198,4 +198,7 @@
 /* Define if plugins enabled */
 #define LLVM_ENABLE_PLUGINS
 
+/* Define if logf128 is available */
+/* #undef LLVM_HAS_LOGF128 */
+
 #endif
